/* Boyd Key Establishment Protocol

- Klt is the long term key shared only between A and B
- pk(Ra) is a fresh public key of A for the run Ra
- K is the session key (short term key)

A -> B: {pk(Ra),B}+Klt
B -> A: {K}*pk(A), {h(K),A}+Klt

NOTE: checking perfect forward secrecy takes about 24 secs.

*/

initiator(A,B,Ra,Klt,Kst,[ 
   start(A,B,init),
   send([pk(Ra),B]+Klt),
   recv([Kst*pk(Ra),[sha(Kst),A]+Klt]),
   run(A,B,init,Ra,Klt,Kst),
   end(A,B,init,Ra,Klt,Kst)
 ]).

responder(A,B,Ra,Klt,Kst,[
   start(B,A,resp),
   recv([pk(Ra),B]+Klt),
   run(B,A,resp,Ra,Klt,Kst),
   send([Kst*pk(Ra),[sha(Kst),A]+Klt]),
   end(B,A,resp,Ra,Klt,Kst)
 ]).

leak(X,[send(X)]).

ik([a,b,e]).


% aliveness of B when A as a responder completes the protocol
scenario([[i1,Init1],[r1,Resp1],[r2,Resp2]],
      all(A,all(B,all(D11,all(D12,all(D13,
         some(A1,some(R1,
            [_,end(A,B,resp,D11,D12,D13)] imp once [_,start(B,A1,R1)])))))))
               ) :-
   initiator(a,b,ra1,kabm,KabS1,Init1),
   responder(a,b,Ra1,kabm,kabs1,Resp1),
   responder(a,b,Ra2,kabm,kabs2,Resp2).

% weak agreement
scenario([[i1,Init1],[r1,Resp1],[r2,Resp2]],
      all(A,all(B,all(D11,all(D12,all(D13,
         some(R1,
            [_,end(A,B,resp,D11,D12,D13)] imp once [_,start(B,A,R1)]))))))
               ) :-
   initiator(a,b,ra1,kabm,KabS1,Init1),
   responder(a,b,Ra1,kabm,kabs1,Resp1),
   responder(a,b,Ra2,kabm,kabs2,Resp2).

% non-injective agreement
scenario([[i1,Init1],[r1,Resp1],[r2,Resp2]],
      all(A,all(B,all(D11,all(D12,all(D13,
            [_,end(A,B,resp,D11,D12,D13)] imp once [_,run(B,A,init,D11,D12,D13)])))))
               ) :-
   initiator(a,b,ra1,kabm,KabS1,Init1),
   responder(a,b,Ra1,kabm,kabs1,Resp1),
   responder(a,b,Ra2,kabm,kabs2,Resp2).

% perfect forward secrecy
scenario([[a,Init1],[b,Resp1],[l,Leak]],
              all(A,all(B,all(N,all(Klt,learn(Klt) and yest (once ([_,end(A,B,init,N,Klt,kst)] and neg learn(kst))) 
                    imp neg learn(kst)))))
             ) :-
   initiator(a,b,na,klt,Kst,Init1),
   responder(a,b,Na,klt,kst,Resp1),
   leak(klt,Leak). 


                         

