/* 
BAN-modified Andrew Secure RPC 
1.   A -> B :   A, {Na}Kab 
2.   B -> A :   {succNa, Nb}Kab 
3.   A -> B :   {succNb}Kab 
4.   B -> A :   {K'ab, N'b, Na}Kab 
*/

initiator(A,B,Na,Nb,KabM,KabS,[ 
   start(A,B,init),
   send([A,Na+KabM]),
   recv([sha(Na),Nb]+KabM),
   send(sha(Nb)+KabM),
   recv([KabS,Na]+KabM),
   end(A,B,init,Na,Nb,KabM,KabS)   % "end" is also the flag for "run"
 ]).

responder(A,B,Na,Nb,KabM,KabS,[
   start(B,A,resp),
   recv([A,Na+KabM]),
   run(B,A,resp,Na,Nb,KabM,KabS),
   send([sha(Na),Nb]+KabM),
   recv(sha(Nb)+KabM),
   send([KabS,Na]+KabM),
   end(B,A,resp,Na,Nb,KabM,KabS)
 ]).

ik([a,b,e]).

% aliveness test (1 initiator and 1 responder)
scenario([[a1,Init1],[b1,Resp1]],
      all(A,all(B,all(D11,all(D12,all(D13,all(D14,
         some(A1,some(R1,
            [_,end(A,B,init,D11,D12,D13,D14)] imp once [_,start(B,A1,R1)]))))))))
               ) :-
   initiator(a,b,na1,Nb1,kabm,KabS1,Init1),
   responder(a,b,Na1,nb1,kabm,kabs1,Resp1).

% weak agreement test (1 initiator and 1 responder)
scenario([[a1,Init1],[b1,Resp1]],
      all(A,all(B,all(D11,all(D12,all(D13,all(D14,
         some(R1,
            [_,end(A,B,init,D11,D12,D13,D14)] imp once [_,run(B,A,R1)])))))))
               ) :-
   initiator(a,b,na1,Nb1,kabm,KabS1,Init1),
   responder(a,b,Na1,nb1,kabm,kabs1,Resp1).

% non-injective agreement test (1 init and 1 resp)
scenario([[a1,Init1],[b1,Resp1]],
      all(A,all(B,all(D11,all(D12,all(D13,all(D14,
            [_,end(A,B,init,D11,D12,D13,D14)] imp once [_,run(B,A,resp,D11,D12,D13,D14)]))))))
               ) :-
   initiator(a,b,na1,Nb1,kabm,KabS1,Init1),
   responder(a,b,Na1,nb1,kabm,kabs1,Resp1).

% aliveness test (2 init, 2 resp)
scenario([[a1,Init1],[b1,Resp1],[a2,Init2],[b2,Resp2]],
      all(A,all(B,all(D11,all(D12,all(D13,all(D14,
         some(A1,some(R1,
            [_,end(A,B,init,D11,D12,D13,D14)] imp once [_,run(B,A1,R1)]))))))))
               ) :-
   initiator(a,b,na1,Nb1,kabm,KabS1,Init1),
   responder(a,b,Na1,nb1,kabm,kabs1,Resp1),
   initiator(a,b,na2,Nb2,kabm,KabS2,Init2),
   responder(a,b,Na2,nb2,kabm,kabs2,Resp2).

% weak agreement test (2 init, 2 resp)
scenario([[a1,Init1],[b1,Resp1],[a2,Init2],[b2,Resp2]],
      all(A,all(B,all(D11,all(D12,all(D13,all(D14,
         some(R1,
            [_,end(A,B,init,D11,D12,D13,D14)] imp once [_,run(B,A,R1)])))))))
               ) :-
   initiator(a,b,na1,Nb1,kabm,KabS1,Init1),
   responder(a,b,Na1,nb1,kabm,kabs1,Resp1),
   initiator(a,b,na2,Nb2,kabm,KabS2,Init2),
   responder(a,b,Na2,nb2,kabm,kabs2,Resp2).

% non-injective agreement test (2 init, 2 resp)
scenario([[a1,Init1],[b1,Resp1],[a2,Init2],[b2,Resp2]],
      all(A,all(B,all(D11,all(D12,all(D13,all(D14,
            [_,end(A,B,init,D11,D12,D13,D14)] imp once [_,run(B,A,resp,D11,D12,D13,D14)]))))))
               ) :-
   initiator(a,b,na1,Nb1,kabm,KabS1,Init1),
   responder(a,b,Na1,nb1,kabm,kabs1,Resp1),
   initiator(a,b,na2,Nb2,kabm,KabS2,Init2),
   responder(a,b,Na2,nb2,kabm,kabs2,Resp2).

